#!/usr/bin/env python
# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006 Serge Weinstock <serge.weinstock@ntlworld.com>>
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
"""
The base class for all the views
"""
from qt import QVBoxLayout, QLabel, QSizePolicy, QFrame, QSplitter, QString, QCursor
from kdecore import i18n, KTempFile, KApplication, KURL
from kdeui import KMessageBox
from kio import KIO
from kfile import KFileDialog
from kmdi import KMdiChildView
from kdeutils import ImageScrollView
from utils import property
import sys, webshots, kdeutils


class BaseView(KMdiChildView):
    """
    The view used for files containing only one picture
    """
    FORBIDDEN_CHAR_IN_PATH = '\/:*?"<>|'
    MAX_LEN_CAPTION = 30
    def __init__(self, url, title, parent):
        """
        Constructor

        @param[in] url a KURL to the original file
        @param[in] title the view title
        @param[in] parent the view parent widget
        """
        if len(title) <= BaseView.MAX_LEN_CAPTION:
            name = title
        else:
            name = i18n("%1...").arg(title[:BaseView.MAX_LEN_CAPTION-3])
        #
        KMdiChildView.__init__(self, name, parent)
        #
        self.url = KURL(url)
        self.maincaption = i18n("%1 - %2").arg(url.filename()).arg(title)
    def createImageViewer(self, pixmap, parent, addLayout):
        """
        Create the image viewer part of the view
        """
        if addLayout:
            layout = QVBoxLayout(parent)
        self.currentPixmap = pixmap
        self.iSplitter = QSplitter(QSplitter.Vertical, parent)
        self.image = ImageScrollView(True, self.iSplitter, pixmap.pixmap)
        self.image.setSizePolicy(QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding))
        self.label_frame = QFrame(self.iSplitter)
        self.label_frame.setFrameShape(QFrame.StyledPanel)
        self.label_frame.setFrameShadow(QFrame.Raised)
        self.label_frame.setMargin(0)
        label_frameLayout = QVBoxLayout(self.label_frame,6,6)
        self.label = QLabel(self.label_frame)
        self.label.setMargin(0)
        self.label.setText(self.getHTMLDesc(pixmap))
        label_frameLayout.addWidget(self.label)
        self.iSplitter.setResizeMode(self.image, QSplitter.Auto)
        self.iSplitter.setResizeMode(self.label_frame, QSplitter.KeepSize)
        if addLayout:
            layout.addWidget(self.iSplitter)
        self.image.setAcceptDrops(True)
        self.label.setAcceptDrops(True)
        self.label_frame.setAcceptDrops(True)
        self.iSplitter.setAcceptDrops(True)
        self.setAcceptDrops(True)

    def getHTMLDesc(self, pixmap, album = ""):
        """
        Build the HTML for an image description

        @param[in] pixmap the webshots.Pixmap we are working on
        @param[in] album the default album name
        """
        return i18n(
"""<div align=center>
<table>
<table bgcolor="#d4d4d4">
   <tr><th bgcolor="#a2c511">Property</th><th bgcolor="#a2c511">Value</th></tr>
   <tr><th bgcolor="#ababab">Title</th><td bgcolor="#d4d4d4">%1</td></tr>
   <tr><th bgcolor="#ababab">Credits</th><td bgcolor="#d4d4d4">%2</td></tr>
   <tr><th bgcolor="#ababab">Collection</th><td bgcolor="#d4d4d4">%3</td></tr>
   <tr><th bgcolor="#ababab">ID</th><td bgcolor="#d4d4d4">%4</td></tr>
   <tr><th bgcolor="#ababab">Size</th><td bgcolor="#d4d4d4">%5 pixels</td></tr>
</table>
</div>
""").arg(pixmap.info.get(webshots.K_TITLE, "")) \
    .arg(pixmap.info.get(webshots.K_CREDITS, "")) \
    .arg(pixmap.info.get(webshots.K_ALBUM, album)) \
    .arg(pixmap.info.get(webshots.K_ID, "")) \
    .arg("%dx%d" % (pixmap.pixmap.width(), pixmap.pixmap.height()))

    @property
    def stretchMode(): #@NoSelf
        """The stretchMode property"""
        def fget(self):
            return self.image.getStretchMode()
        def fset(self, stretch):
            self.image.setStretchMode(stretch)
    def setImage(self, pixmap):
        """
        Change the displayed image

        @param[in] pixmap the webshots.Pixmap to display
        """
        self.currentPixmap = pixmap
        self.label.setText(self.getHTMLDesc(pixmap))
        self.image.setImage(pixmap.pixmap)

    def saveImage(self, path, format, quality = -1):
        """
        Save the currently selected image

        @param[in] path the output path
        @param[in] format the image format ("JPEG", "PNG", "BMP", "PPM", ...)
        @param[in] quality the quality of the ouput (0 to 100 or -1 for defaults)
        @return True if successfull
        """
        return self.image.saveImage(path, format, quality)

    def saveJPEG(self, path, pixmap = None):
        """
        Save the JPEG bytes of the currently selected image
        @param[in] path the output path
        @param[in] pixmap The pixmap to save. if None the currently selected pixmap is saved
        """
        if not pixmap: pixmap = self.currentPixmap
        f = open(unicode(path), "wb")
        pixmap.jpeg.tofile(f)
    def saveCurrentImage(self):
        """
        save the current image
        """
        cfg = KApplication.kApplication().cfg.mainframe
        filter = QString("*.jpg *.JPG *.jpeg *.JPEG|")
        filter += i18n("JPEG Image")
        filter += " (*.jpg *.jpeg)"
        ipath = (self.currentPixmap.info.get(webshots.K_TITLE, "image")+".jpg")
        for c in BaseView.FORBIDDEN_CHAR_IN_PATH:
            ipath = ipath.replace(c, "_")
        if cfg.lastSaveDir:
            ldir = KURL(cfg.lastSaveDir)
            ldir.setFileName(ipath)
            ipath = ldir.prettyURL()
        url = KFileDialog.getSaveURL(
            ipath,
            filter,
            self,
            i18n("Save Image")
            )
        if url.isEmpty():
            return
        try:
            if KIO.NetAccess.exists(url, False, self):
                if KMessageBox.warningContinueCancel(
                      self,
                      i18n("A file named '%1' already exists.  Are you sure you want to overwrite it?").arg(url.prettyURL()),
                      i18n("Overwrite File?")
                      ) != KMessageBox.Continue:
                    return
            if not url.isValid():
                KMessageBox.sorry(self, i18n("Malformed URL"))
                return
            try:
                KApplication.kApplication().setOverrideCursor(QCursor.waitCursor)
                KApplication.kApplication().mainWidget().setDisabled(True)
                if url.isLocalFile():
                    self.saveJPEG(url.path())
                    cfg.lastSaveDir = url.upURL().prettyURL()
                    return
                tmpFile = KTempFile()
                tmpFile.setAutoDelete(True);
                self.saveJPEG(tmpFile.name())
            finally:
                KApplication.kApplication().restoreOverrideCursor()
                KApplication.kApplication().mainWidget().setDisabled(False)
            if not KIO.NetAccess.upload(tmpFile.name(), url, self):
                KMessageBox.error(self, i18n("Could not save remote file"))
                return
            cfg.lastSaveDir = url.upURL().prettyURL()
        except:
            error = sys.exc_info()
            kdeutils.detailedError(self, i18n("Could not save file:\n'%1'").arg(url.prettyURL()), error)
    def saveAllImages(self):
        """
        save all the images
        """
        # just a dummy function that will be overidden by view dealing with collections
        pass
