#!/usr/bin/env python
# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006 Serge Weinstock <serge.weinstock@ntlworld.com>>
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
"""
The view used for files containing collections of pictures
"""
from baseview import BaseView
from kdecore import i18n, KApplication, KTempFile, KURL
from kdeui import KListBox, KMessageBox, KStdGuiItem
from kdeutils import getIcon, ListBoxTextEx, ImageScrollView
from kfile import KFileDialog, KFile
from kio import KIO
from qt import QVBoxLayout, QSplitter, SIGNAL, QString, QCheckListItem, QCursor
import sys, filelist, webshots, kdeutils

class SaveAllFilesDialog(KFileDialog):
    """
    A select directory dialog with a list of all the files to save
    The user canc check or unckeck these files
    """
    class FileListItem(QCheckListItem):
        '''
        A specialized QCheckListItem for displaying a file to save
        '''
        def __init__(self, fileData, parent, stateChangeSlot):
            '''
            Constructor
            @param[in] fileData all the info about the file to show: [To_Save?, size_in_pixels_string, filename]
            @param[in] parent the parent widget
            @param[in] stateChangeSlot the slot to call when the item state changes
            '''
            QCheckListItem.__init__(self, parent, "", QCheckListItem.CheckBox)
            self.do_signal = False
            self.fileData = fileData
            self.setOn(self.fileData[0])
            self.setText(0, self.fileData[1])
            self.setText(1, self.fileData[2])
            self.stateChangeSlot = stateChangeSlot
            self.do_signal = True
        def stateChange(self, newState):
            '''
            This is called when the user clicks on the checkbox

            @param[in] newState set to True if the item is now checked
            '''
            self.fileData[0] = newState
            if self.do_signal:
                self.stateChangeSlot(newState)
    def __init__(self, files, startDir, filter, parent):
        '''
        Constructor
        @param[in] files the list of [To_Save?, size_in_pixels_string, filename] to show in the list of file to save
        @param[in] startDir:
        @param[in] filter:
        @param[in] parent:
        '''
        self.form = filelist.FileList()
        KFileDialog.__init__(self, startDir, filter, parent, "file_save_all", True, self.form)
        # look for the layout widget added by pyuic
        #self.pix_layout = QVBoxLayout(self.form.pixPreviewFrm,6,6)
        for w in self.form.pixPreviewFrm.children():
            if isinstance(w, QVBoxLayout):
                self.pix_layout = w
                break
        self.pixPreview = ImageScrollView(True, self.form.pixPreviewFrm, None)
        self.pix_layout.addWidget(self.pixPreview)
        self.setMode(KFile.Directory)
        self.connect(self.form.btnSelAll,SIGNAL("clicked()"),self.onSelectAll)
        self.connect(self.form.btnSelNone,SIGNAL("clicked()"),self.onSelectNone)
        self.connect(self.form.btnSelInv,SIGNAL("clicked()"),self.onSelectInv)
        self.connect(self.form.listFiles,SIGNAL("selectionChanged (QListViewItem*)"),self.onSelChange)
        self.N = 0
        for f in files:
            self.N += int(f[0])
            SaveAllFilesDialog.FileListItem(f, self.form.listFiles, self.itemStateChange)
    def onSelectAll(self):
        '''
        Slot for selecting all files
        '''
        li = self.form.listFiles.firstChild()
        while li:
            li.setOn(True)
            li = li.nextSibling()
    def onSelectNone(self):
        '''
        Slot for selecting no file
        '''
        li = self.form.listFiles.firstChild()
        while li:
            li.setOn(False)
            li = li.nextSibling()
    def onSelectInv(self):
        '''
        Slot for inverting the selection of files to save
        '''
        li = self.form.listFiles.firstChild()
        while li:
            li.setOn(not li.isOn())
            li = li.nextSibling()
    def itemStateChange(self, newState):
        if newState:
            self.N += 1
        else:
            self.N -= 1
        self.okButton().setEnabled(self.N != 0)
    def onSelChange(self, item):
        pass
        self.pixPreview.setImage(item.fileData[3].pixmap)

class CollectionView(BaseView):
    """
    The view used for files containing collections of pictures
    """
    __count = 0
    def __init__(self, url, title, pixmaps, parent):
        """
        Constructor
        @param[in] url a KURL to the original file
        @param[in] pixmap the QPixmap containing the image
        @param[in] info the info about the image
        @param[in] parent the view parent widget
        """
        if not title:
            title = i18n("Collection %1").arg(CollectionView.__count+1)
        #
        BaseView.__init__(self, url, title, parent)
        #
        self.pixmaps = pixmaps
        self.setIcon(getIcon("webshot-col"))
        # let's create the the UI
        layout = QVBoxLayout(self)
        self.hSplitter = QSplitter(QSplitter.Horizontal, self, "hSplitter")

        self.list_images = KListBox(self.hSplitter,"list_images")

        self.createImageViewer(self.pixmaps[0], self.hSplitter, False)
        #
        i = 0
        for pixmap in self.pixmaps:
            name = pixmap.info.get(webshots.K_TITLE, i18n("Image %1").arg(i+1))
            self.list_images.insertItem(ListBoxTextEx(name, i))
            i += 1
        self.list_images.setSelected(0, True)
        self.connect(self.list_images, SIGNAL("selectionChanged(QListBoxItem *)"), self.imageSelected)
        #
        self.hSplitter.setResizeMode(self.list_images, QSplitter.KeepSize)
        self.hSplitter.setResizeMode(self.iSplitter, QSplitter.Auto)
        layout.addWidget(self.hSplitter)
        #
        CollectionView.__count += 1
        #

    def imageSelected(self, qlbi):
        """
        slot called when the user select an iten in the image list

        @param[in] qlbi the QListBoxItem of the selecting item
        """
        self.setImage(self.pixmaps[qlbi.data])

    def saveAllImages(self):
        """
        save all the images
        """
        # reset the don't aks again flag for KMessageBox.warningContinueCancel
        cfg = KApplication.kApplication().config()
        cfg.setGroup("Notification Messages")
        cfg.writeEntry("saveAllprompt", True)
        #
        cfg = KApplication.kApplication().cfg.mainframe
        files = []
        for p in self.pixmaps:
            fn = p.info.get(webshots.K_TITLE, "image")
            fn += ".jpg"
            for c in BaseView.FORBIDDEN_CHAR_IN_PATH:
                fn = fn.replace(c, "_")
            sz = i18n("%1x%2").arg(p.pixmap.width()).arg(p.pixmap.height())
            files.append([True, sz, fn, p]) # checked, size, filename, pixmap
        dlg = SaveAllFilesDialog(files, cfg.lastSaveDir, QString.null, self)
        dlg.exec_loop()
        urlBase = dlg.selectedURL()
        if urlBase.isEmpty():
            return
        for f in files:
            if not f[0]:
                continue
            try:
                url = KURL(urlBase, f[2])
                if KIO.NetAccess.exists(url, False, self):
                    if KMessageBox.warningContinueCancel(
                          self,
                          i18n("A file named '%1' already exists.  Are you sure you want to overwrite it?").arg(url.prettyURL()),
                          i18n("Overwrite File?"),
                          KStdGuiItem.cont(),
                          "saveAllprompt"
                          ) != KMessageBox.Continue:
                        return
                if not url.isValid():
                    KMessageBox.sorry(self, i18n("Malformed URL"))
                    return
                try:
                    KApplication.kApplication().setOverrideCursor(QCursor.waitCursor)
                    KApplication.kApplication().mainWidget().setDisabled(True)
                    if url.isLocalFile():
                        self.saveJPEG(url.path(), f[3])
                        continue
                    tmpFile = KTempFile()
                    tmpFile.setAutoDelete(True);
                    self.saveJPEG(tmpFile.name(), f[3])
                finally:
                    KApplication.kApplication().mainWidget().setDisabled(False)
                    KApplication.kApplication().restoreOverrideCursor()
                if not KIO.NetAccess.upload(tmpFile.name(), url, self):
                    KMessageBox.error(self, i18n("Could not save remote file"))
                    return
            except:
                error = sys.exc_info()
                kdeutils.detailedError(self, i18n("Could not save file:\n'%1'").arg(url.prettyURL()), error)
        cfg.lastSaveDir = url.upURL().prettyURL()
