#!/usr/bin/env python
# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006 Serge Weinstock <serge.weinstock@ntlworld.com>
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
"""
Utilities for KDE/Qt
"""
from qt import QScrollView, QLabel, QListBoxText, SIGNAL, PYSIGNAL, QWidget, QCustomEvent
from kdecore import KIcon, KGlobal, KApplication
from kdeui import KMessageBox
from kio import KDirLister

def detailedError(parent, message, exc_info):
    """
    Show an error message box with a "Details >>" button.
    The details report the text of the error and the contents of the stack when the error occured

    @param[in] parent the parent of the message box
    @param[in] message the main message
    @param[in] exc_info the info about the exception to report in the details (usually the contents of sys.exc_info())
    """
    from traceback import print_exception
    from StringIO import StringIO
    out = StringIO()
    print_exception(exc_info[0], exc_info[1], exc_info[2], None, out)
    KMessageBox.detailedError(parent, message, out.getvalue())

def getIcon(name, group=KIcon.NoGroup, size=KIcon.SizeSmall):
    """
    returns KDE kde icon by name

    @param[in] name the icon name
    @param[in] group the icon group
    @param[in] size the icon size
    """
    return KGlobal.instance().iconLoader().loadIcon(name, group, size)

__next_event_ID = QCustomEvent.User
def getCustomEventID():
    """
    Return an ID for a custom event
    """
    global __next_event_ID
    __next_event_ID += 1
    return __next_event_ID

class ImageScrollView(QScrollView):
    """
    This class is a scrollview with contains a label displaying an image.

    If stretchMode is True, the size of the label is adjusted to the the size of the scrollview viewport
    and the displayed image is stretched

    If stretchMode is False, the size of the label is the real size of the image and no stretching is done
    """
    def __init__(self, stretch, parent, pixmap):
        """
        Constructor

        @param[in] stretch if True the image will be stretched
        @param[in] parent the parent of this widget
        @param[in] pixmap the image to display
        """
        QScrollView.__init__(self, parent)
        self.image = QLabel(self.viewport())
        if (not pixmap) or (not pixmap.width()):
            self.ratio = 1
        else:
            self.ratio = float(pixmap.height())/pixmap.width()
        if pixmap:
            self.image.setPixmap(pixmap)
        self.addChild(self.image)
        self.setStretchMode(stretch, True)
    def __resize(self):
        """
        Do the actual resizing in stretch mode
        """
        if self.stretchMode:
            vsz = self.viewportSize(self.width(), self.height())
            w = vsz.width()
            h = int(w * self.ratio)
            if h > vsz.height():
                h = vsz.height()
                w = int(h / self.ratio)
            self.resizeContents(vsz.width(), vsz.height())
            self.image.setGeometry((vsz.width() - w)/2, (vsz.height() - h)/2, w, h)
    def resizeEvent(self, event):
        """
        This event handler is called when the scrollview has been resized

        @param[in] event the QResizeEvent event
        """
        QScrollView.resizeEvent(self, event)
        self.__resize()
    def setStretchMode(self, stretch, force = False):
        """
        Set the widget stretch mode

        @param[in] stretch if True the image will be stretched
        @param[in] force if True force all the changes even if the widget is already set for this mode
        """
        if (not force) and (stretch == self.stretchMode):
            return
        self.stretchMode = stretch
        self.image.setScaledContents(stretch)
        if self.stretchMode:
            self.setResizePolicy(QScrollView.Manual)
            self.setContentsPos(0, 0)
            self.setHScrollBarMode(QScrollView.AlwaysOff)
            self.setVScrollBarMode(QScrollView.AlwaysOff)
        else:
            self.setResizePolicy(QScrollView.AutoOne)
            self.setHScrollBarMode(QScrollView.Auto)
            self.setVScrollBarMode(QScrollView.Auto)
            sh = self.image.sizeHint()
            self.resizeContents(sh.width(), sh.height())
            self.image.setGeometry(0, 0, sh.width(), sh.height())
        self.__resize()
    def getStretchMode(self):
        """
        Returns True if the widget is in "stretch mode"
        """
        return self.stretchMode
    def setImage(self, pixmap):
        """
        Set the image displayed in the control

        @param[in] pixmap the image to display
        """
        if (not pixmap) or (not pixmap.width()):
            self.ratio = 1
        else:
            self.ratio = float(pixmap.height())/pixmap.width()
        if pixmap:
            self.image.setPixmap(pixmap)
        self.__resize()
    def saveImage(self, path, format, quality = -1):
        """
        Save the currently selected image

        @param[in] path the output path
        @param[in] format the image format
        @param[in] quality the quality of the ouput (0 to 100 or -1 for defaults)
        @return True if successfull
        """
        return self.image.pixmap().save(path, format, quality)

    def setAcceptDrops(self, acceptDrops):
        QScrollView.setAcceptDrops(self, acceptDrops)
        self.viewport().setAcceptDrops(acceptDrops)
        self.image.setAcceptDrops(acceptDrops)

class ListBoxTextEx(QListBoxText):
    """
    A text listbox item wich keeps track of additional data
    """
    def __init__(self, text, data):
        QListBoxText.__init__(self, text)
        self.data = data

class DirLister(QWidget):
    '''
    Wrapper around the KDirLister class.

    It allows to get recursively the contents of an URL
    as pykde does not provided a usable wrapper for kio::listRecursive

    The main difficulties here are:
    - no usable wrapper is provided by pykde for the items paramaters of the newItem signal
    - KDirLister @b must finish it works before scanning another URL. So we need to keep a list of subfolders and
     only process them after the call to openURL is finished
    '''

    EVENT_OPEN_URL = getCustomEventID()
    def __init__(self, filter = None):
        '''
        Constructor
        @param[in] filter an optional filter for files (@see KDirLister::setNameFilter)
        '''
        QWidget.__init__(self)
        self.dirLister = KDirLister()
        self.dirLister.setMainWindow(KApplication.kApplication().mainWidget())
        self.filter = filter
        if self.filter: self.dirLister.setNameFilter(self.filter)
        self.connect(self.dirLister, SIGNAL("completed ()"), self.__completed)
        self.connect(self.dirLister, SIGNAL("canceled ()"), self.__canceled)
        self.url = None
        self.recursive = True
        self.todo = []
    def openURL(self, url, recursive):
        '''
        list all the files in a URL
        @param[in] url the URL to scan
        @param[in] recursive if True scan also subfolders
        '''
        self.url = KURL(url)
        self.recursive = recursive
        self.todo = []
        self.dirLister.openURL(url)
    def __canceled(self):
        '''
        slot for the signal KDirLister.canceled()
        '''
        self.emit(PYSIGNAL("completed"), (False,))
    def __completed(self):
        '''
        slot for the signal KDirLister.completed()
        '''
        # let's update our list of subfolders not listed
        for l in self.dirLister.items(KDirLister.AllItems):
            if self.recursive and l.isDir():
                self.todo.append(KURL(l.url()))
        # notify that we have some new items
        self.emit(PYSIGNAL("newItems"),(self.dirLister.items(KDirLister.FilteredItems),))
        # if there are some subfolder to process
        # do it asynchronously, by posting an event that will be processed after we exit of the slot
        if self.todo:
            event = QCustomEvent(self.EVENT_OPEN_URL)
            KApplication.kApplication().postEvent(self, event)
            return
        # no more subfolders to process, the task is completed
        self.emit(PYSIGNAL("completed"), (True,))

    def customEvent(self, event):
        """
        This is the event handler for custom events

        @param[in] event the QCustomEvent event
        """
        if event.type() == self.EVENT_OPEN_URL:
            if self.todo:
                # at this stage we have finished with the last call to KDirLister.openURL
                # we can launch another one
                self.dirLister.openURL(self.todo.pop())

# There is a problem with some pykde distributions including mine
# where entries(KIO::Job*, const KIO::UDSEntryList&) returns a NotImplemented instance for the list
# until this problem is fixed, we can't use this code
from qt import QObject
from kdecore import KURL
from kio import KIO, KFileItem
class DirListerEx(QObject):
    def __init__(self, url):
        self.url  = KURL(url)
    def openURL(self, url, recursive): #@UnusedVariable
        self.url = KURL(url)
        job = KIO.listRecursive(self.url, False)
        self.connect(job, SIGNAL("entries(KIO::Job*, const KIO::UDSEntryList&)"), self.__newItems)
        self.connect(job, SIGNAL("result(KIO::Job*)"), self.__result)
    def __newItems(self, job, list): #@UnusedVariable
         newFiles = []
         print list
         for udsList in list:
             f = KFileItem(udsList, self.url, False, True)
             newFiles.append(f)
             self.files.append(f)
         self.emit(PYSIGNAL("newItems"),(newFiles,))
    def __result(self):
        self.emit(PYSIGNAL("completed"), (True,))
