#!/usr/bin/env python
# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006 Serge Weinstock <serge.weinstock@ntlworld.com>
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
"""
This module allow to handle easily kio jobs
"""
from qt import QString, QCustomEvent, QFile, SIGNAL
from kdecore import KApplication, i18n, KTempFile, KURL
from kdeui import KProgressDialog
from kio import KIO
from kdeutils import getCustomEventID
import os

class ContinueJobEvent(QCustomEvent):
    '''
    An event for continuing the current job
    '''
    ID = getCustomEventID()
    def __init__(self):
        '''
        Constructor
        '''
        QCustomEvent.__init__(self, self.ID)

class ProgressEvent(QCustomEvent):
    '''
    An event for updating the sprogress dialog
    '''
    ID = getCustomEventID()
    def __init__(self, steps, totalsteps, text):
        '''
        Constructor
        @param[in] steps the current number of steps for the dialog
        @param[in] maxsteps the maximum number of steps for the dialog
        @param[in] text the progress message
        '''
        QCustomEvent.__init__(self, self.ID)
        self.steps = steps
        self.totalsteps = totalsteps
        self.text = text

class EndJobEvent(QCustomEvent):
    '''
    An event for signalling the end of the job
    '''
    ID = getCustomEventID()
    def __init__(self, results):
        '''
        Constructor
        @param[in] results the results of the job
        '''
        QCustomEvent.__init__(self, self.ID)
        self.results = results

class Job(object):
    """
    This is the class that needs to be derived for achieving some work.

    It is a @b generator that @b yield values for communicating with the upper layer.

    The upper layers runs the generator until it yields a value. This value is used by the upper layer
    for doing some work. Once this work is done, execution is resumed in the generator.

    If the yielded value is:
    - a kio.Job, the job is run until completion
    - a kio.Job.KioRes, the value is used as the result of a "fake" kio.Job
    - a ProgressEvent (by using setProgress) for updating the progress dialog
    """
    class CancelException(Exception):
        '''
        An exception thrwon when the job is cancelled
        '''
        pass
    class KioRes(object):
        """
        all the info about the current kio job
        """
        def __init__(self, error = 0, errorString = QString()):
            self.error = error                # last job error code
            self.errorString = errorString    # last job error string
            self.res = None                   # result of last job
    def __init__(self, dialog = None):
        """
        Constructor
        @param dialog The progress dialog
        """
        self.results = None         # the job esults
        self.cancelled = False      # cancel flags
        self.dialog = dialog        # the progress dialog
        self.kio = Job.KioRes()     # all about the last kio job
        self.tmpfiles = []          # the list of temporary files
    def setDialog(self, dialog):
        '''
        Set the progress dialog
        @param dialog the progress dialog
        '''
        self.dialog = dialog
    def cancel(self):
        '''
        set the job cancel flag to True
        '''
        self.cancelled = True
    def isCancelled(self):
        '''
        return True if the job cancel flags has been set
        '''
        return self.cancelled
    def checkCancelled(self):
        '''
        check if the job cancel flag has been set to True.
        If so it raises a CancelException.
        '''
        if self.cancelled:
            raise Job.CancelException()
    def setProgress(self, steps = None, totalsteps = None, text = None):
        '''
        @param[in] steps the current number of steps for the dialog
        @param[in] maxsteps the maximum number of steps for the dialog
        @param[in] text the progress message
        @return a value that can be used for updating the progress dialog
        '''
        return ProgressEvent(steps, totalsteps, text)
    def kio_download(self, src, target = QString()):
        '''
        Downloads a file from an arbitrary URL (src) to a temporary file on the local filesystem (target).
        @param[in] src URL Reference to the file to download.
        @param[in] target a QString containing the final local location of the file.
            If you insert an empty string, it will return a location in a temporary spot
        @return a value that can be used for launching the job
        '''
        res = self.kio = Job.KioRes()
        if src.isLocalFile():
            # file protocol. We do not need the network
            res.res = src.path()
            accessible = os.access(unicode(src.path()), os.R_OK);
            if not accessible:
                res.errorString = i18n("File '%1' is not readable").arg(target);
                res.error = KIO.ERR_COULD_NOT_READ
            return res
        if target.isEmpty():
            tmpFile = KTempFile()
            tmpFile.setAutoDelete(True)
            self.tmpfiles.append(tmpFile)
            target = tmpFile.name()
        res.res = target
        target = KURL(target)
        return KIO.file_copy(src, target, -1, True, False)
    def kio_upload(self, src, target):
        '''
        Uploads file src to URL target.
        @param[in] src URL Referencing the file to upload
        @param[in] target URL containing the final location of the file
        @return a value that can be used for launching the job
        '''
        res = self.kio = Job.KioRes()
        if target.isEmpty():
            res.errorString = i18n("No target defined");
            res.error = KIO.ERR_COULD_NOT_READ
            return res
        if target.isLocalFile() and target.path() == src:
            return res
        res.res = target
        src = KURL(src)
        job = KIO.file_copy(src, target, -1, True, False)
        job.setInteractive(False)
        return job
    def kio_exists(self, url, source = False):
        '''
        Tests whether a URL exists.
        @param[in] url the URL to test
        @param[in] source if True the URL is a source (we want to read it)  or a destination (we want to write it)
        '''
        res = self.kio = Job.KioRes()
        if url.isLocalFile():
            res.res = QFile.exists(url.path())
            return res
        job = KIO.stat(url, not url.isLocalFile())
        job.setDetails(0)
        job.setSide(source)
        job.setInteractive(False)
        return job
    def kio_removeTempFile(self, name):
        '''
        Removes the specified file if and only if it was created by kio_xxx as a temporary file for a former download.
        @param[in] name path to temporary file to remove. May not be empty.
        '''
        res = self.kio = Job.KioRes()
        tf = None
        for tf in self.tmpfiles:
            if tf.name() == name:
                try:
                    tf.unlink()
                    self.tmpfiles.remove(tf)
                    del tf
                    return res
                except:
                    return res
        return res
    def run(self):
        '''
        The job main routine
        '''
        # must be overriden
        assert(False)

class ProgressDialog(KProgressDialog):
    """
    A KProcessDialog that can handle a Job
    """
    def __init__(self, job, owner, parent, name, caption, text, cancelButtonText, autoClose):
        """
        Constructor
        @param[in] job the job to run
        @param[in] owner the object to notify about the end of the job
        @param[in] parent parent of the progress dialog
        @param[in] name name of the progress dialog
        @param[in] caption text to display in window title bar
        @param[in] text text to display in the dialog
        @param[in] cancelButtonText text to show for the cancel button or
            None to disallow cancellation and to hide the cancel button
        @param[in] autoClose set to True, if the dialog should close itself on completion
        """
        KProgressDialog.__init__(self, parent, name, caption, text, True)
        self.kiojob = None            # the current kio job
        self.job = job                # the job to do
        self.gen = job.run()          # the generator
        job.setDialog(self)
        self.owner = owner            # the object to notify about the end of the job
        self.autoClose = autoClose    # close the dialog once finished ?
        self.results = [False, None]  # the results of the job
        if cancelButtonText:
            self.setButtonText(cancelButtonText)
        else:
            self.setAllowCancel(False)
        self.connect(self, SIGNAL("cancelClicked()"), self.onCancel)
        # let's go
        self.continueJob()
    def __run(self):
        """
        The main routine for handling the job
        """
        try:
            res = self.gen.next()
            if isinstance(res, ProgressEvent):
                KApplication.kApplication().postEvent(self, res)
                return
            if isinstance(res, KIO.Job):
                # it's a kio::Job. Let's wait until it's finished
                self.kiojob = res
                self.kiojob.setWindow(self)
                self.connect(res, SIGNAL("result(KIO::Job *)"), self.kioResult);
                return
            if isinstance(res, Job.KioRes):
                self.job.kio = res
                self.continueJob()
                return
            raise Exception("Invalid kiojob.Job return type: %s (%s)" % (str(res.__class__), str(res)))
        except Job.CancelException:
            # the job has been cancelled
            self.results = [False, None]
        except StopIteration:
            # the job is finished
            self.results = [True, self.job.results]
            # let's be sure to display 100%
            self.progressBar().setProgress(self.progressBar().totalSteps())
            self.disconnect(self, SIGNAL("cancelClicked()"), self.onCancel)
            # close the dialog?
            if self.autoClose:
                self.close()
        except:
            # unknown error
            self.results = [False, None]
            # debugging
            from traceback import print_exception
            from StringIO import StringIO
            import sys
            exc_info = sys.exc_info()
            out = StringIO()
            print_exception(exc_info[0], exc_info[1], exc_info[2], None, out)
            print >>sys.stderr, out.getvalue()
        if self.owner:
            KApplication.kApplication().postEvent(self.owner, EndJobEvent(self.results))
    def continueJob(self):
        '''
        Send an event for continuing the current job once the current event has been processed
        '''
        KApplication.kApplication().postEvent(self, ContinueJobEvent())
    def kioResult(self, kiojob):
        """
        Process the results of KIO.Job
        """
        assert(kiojob == self.kiojob)
        self.job.kio.error = kiojob.error()
        self.job.kio.errorString = kiojob.errorString()
        if isinstance(kiojob, KIO.StatJob):
            # no error means that the url exists
            self.job.kio.res = (kiojob.error() == 0)
        self.continueJob()
    def onCancel(self):
        '''
        Called when the user clicks on the cancel button
        '''
        # let's signal that the job is cancelled
        self.job.cancel()
        # abort the kio job if any
        if self.kiojob:
            self.kiojob.kill()
        # allow the job to do some cleanup
        self.continueJob()
    def customEvent(self, event):
        """
        This is the event handler for custom events
        @param[in] event the QCustomEvent event
        """
        if event.type() == ContinueJobEvent.ID:
           self.__run()
        elif event.type() == ProgressEvent.ID:
            if event.text is not None:
                self.progressBar().setLabel(event.text)
            if event.totalsteps is not None:
                self.progressBar().setTotalSteps(event.totalsteps)
            if event.steps is not None:
                self.progressBar().setProgress(event.steps)
            # let's the UI the time to refresh before continuing
            self.continueJob()

def runJob(job, owner, parent, caption, text, cancelButtonText = None, autoClose = False, name = None):
    """
    Launch a job
    @param[in] job the job to run
    @param[in] owner the object to notify about the end of the job
    @param[in] parent parent of the progress dialog
    @param[in] caption text to display in window title bar
    @param[in] text text to display in the dialog
    @param[in] cancelButtonText text to show for the cancel button or
        None to disallow cancellation and to hide the cancel button
    @param[in] autoClose set to True, if the dialog should close itself on completion
    @param[in] name name of the progress dialog
    @return [success, event] where success is set to True if the job completed successfully and event to the resulting TaskFinishedEvent
    """
    dlg = ProgressDialog(job, owner, parent, name, caption, text, cancelButtonText, autoClose)
    dlg.exec_loop()
    return dlg.results
